@@ -19,16 +19,15 @@ import itertools
 import logging
 import operator
 import os
-import shutil
 import subprocess
 import sys
 import tempfile
 from urllib import parse
-from uuid import uuid4
 
 import daiquiri
 
 from git_pull_request import pagure
+from git_pull_request import textparse
 
 import github
 
@@ -185,6 +184,8 @@ def get_hosttype_hostname_user_repo_from_url(url):
     else:
         path = parsed.path[1:].rstrip('/')
         host = parsed.netloc
+        if "@" in host:
+            username, sep, host = host.partition("@")
     hosttype = get_hosttype(host)
     if hosttype == "pagure":
         user, repo = None, path
@@ -198,6 +199,7 @@ def split_and_remove_empty_lines(s):
 
 
 def parse_pr_message(message):
+    message = textparse.remove_ignore_marker(message)
     message_by_line = message.split("\n")
     if len(message) == 0:
         return None, None
@@ -215,11 +217,21 @@ def git_get_commit_body(commit):
 
 def git_get_log_titles(begin, end):
     log = _run_shell_command(
-        ["git", "log", "--format=%s", "%s..%s" % (begin, end)],
+        ["git", "log", "--no-merges", "--format=%s", "%s..%s" % (begin, end)],
         output=True)
     return list(split_and_remove_empty_lines(log))
 
 
+def git_get_log(begin, end):
+    return _run_shell_command(
+        ["git", "log",
+         "--no-merges",
+         "--reverse",
+         "--format=## %s%n%n%b",
+         "%s..%s" % (begin, end)],
+        output=True)
+
+
 def git_get_title_and_message(begin, end):
     """Get title and message summary for patches between 2 commits.
 
@@ -228,29 +240,30 @@ def git_get_title_and_message(begin, end):
     :return: number of commits, title, message
     """
     titles = git_get_log_titles(begin, end)
-    title = "Pull request for " + end
     if len(titles) == 1:
         title = titles[0]
-    pr_template = find_pull_request_template()
-    if pr_template:
-        message = get_pr_template_message(pr_template)
     else:
-        if len(titles) == 1:
-            message = git_get_commit_body(end)
-        else:
-            message = "\n".join(titles)
+        title = "Pull request for " + end
 
-    return (len(titles), title, message)
+    pr_template = get_pull_request_template()
+    if pr_template is not None:
+        message = textparse.concat_with_ignore_marker(
+            pr_template, git_get_log(begin, end)
+        )
+    elif len(titles) == 1:
+        message = git_get_commit_body(end)
+    else:
+        message = git_get_log(begin, end)
+
+    return len(titles), title, message
 
 
 def git_pull_request(target_remote=None, target_branch=None,
                      title=None, message=None,
                      comment=None,
                      rebase=True,
-                     force_editor=False,
                      download=None,
                      download_setup=False,
-                     tag_previous_revision=False,
                      fork=True,
                      setup_only=False,
                      branch_prefix=None,
@@ -326,7 +339,7 @@ def git_pull_request(target_remote=None, target_branch=None,
     else:
         retcode = fork_and_push_pull_request(
             g, hosttype, repo, rebase, target_remote, target_branch, branch,
-            user, title, message, comment, force_editor, tag_previous_revision,
+            user, title, message, comment,
             fork, setup_only, branch_prefix, dry_run, labels,
         )
 
@@ -382,7 +395,7 @@ def edit_file_get_content_and_remove(filename):
     return content
 
 
-def find_pull_request_template():
+def get_pull_request_template():
     filename = "PULL_REQUEST_TEMPLATE*"
     pr_template_paths = [
         filename,
@@ -395,16 +408,8 @@ def find_pull_request_template():
     for path in pr_template_paths:
         templates = glob.glob(path)
         if templates:
-            return templates[0]
-
-
-def get_pr_template_message(template):
-    fd, bodyfilename = tempfile.mkstemp()
-    os.close(fd)
-    shutil.copy(template, bodyfilename)
-    content = edit_file_get_content_and_remove(bodyfilename)
-
-    return content
+            with open(templates[0]) as t:
+                return t.read()
 
 
 def edit_title_and_message(title, message):
@@ -418,23 +423,10 @@ def edit_title_and_message(title, message):
     return parse_pr_message(content)
 
 
-def preserve_older_revision(branch, remote_to_push):
-    tag = "{}-{}".format(branch, uuid4())
-    _run_shell_command(["git", "tag", tag])
-    try:
-        _run_shell_command(["git", "push", remote_to_push, tag])
-    except RuntimeError:
-        LOG.error("Unable to push tag %s for previous revision", tag)
-    else:
-        LOG.info("Previous revision of patchset saved on tag %s", tag)
-
-    _run_shell_command(["git", "tag", "-d", tag])
-
-
 def fork_and_push_pull_request(g, hosttype, repo_to_fork, rebase,
                                target_remote, target_branch, branch, user,
                                title, message, comment,
-                               force_editor, tag_previous_revision, fork,
+                               fork,
                                setup_only, branch_prefix,
                                dry_run=False,
                                labels=None):
@@ -527,60 +519,94 @@ def fork_and_push_pull_request(g, hosttype, repo_to_fork, rebase,
 
     pulls = list(repo_to_fork.get_pulls(base=target_branch,
                                         head=head))
+
+    nb_commits, git_title, git_message = git_get_title_and_message(
+        "%s/%s" % (target_remote, target_branch), branch)
+
     if pulls:
         for pull in pulls:
-            if dry_run:
-                LOG.info("Pull-request would be updated:\n  %s", pull.html_url)
-                continue
+            if title is None:
+                # If there's only one commit, it's very likely the new PR title
+                # should be the actual current title. Otherwise, it's unlikely
+                # the title we autogenerate is going to be better than one
+                # might be in place now, so keep it.
+                if nb_commits == 1:
+                    ptitle = git_title
+                else:
+                    ptitle = pull.title
+            else:
+                ptitle = title
 
-            LOG.info("Pull-request updated:\n  %s", pull.html_url)
+            body = textparse.concat_with_ignore_marker(
+                message or git_message,
+                ">\n> Current pull request content:\n" +
+                pull.title + "\n\n" + pull.body,
+            )
 
-            if force_editor:
-                title, message = edit_title_and_message(title or pull.title,
-                                                        message or pull.body)
+            ptitle, body = edit_title_and_message(ptitle, body)
 
-            if title and message:
-                pull.edit(title=title, body=message)
-                LOG.debug("Updated pull-request title and message")
-            elif title:
-                pull.edit(title=title)
-                LOG.debug("Updated pull-request title")
-            elif message:
-                pull.edit(body=message)
-                LOG.debug("Updated pull-request message")
+            if ptitle is None:
+                LOG.critical("Pull-request message is empty, aborting")
+                return 40
+
+            if ptitle and body:
+                if dry_run:
+                    LOG.info("Would edit title and body")
+                    LOG.info("%s\n", ptitle)
+                    LOG.info("%s", body)
+                else:
+                    pull.edit(title=ptitle, body=body)
+                    LOG.debug("Updated pull-request title and body")
+            elif ptitle:
+                if dry_run:
+                    LOG.info("Would edit title")
+                    LOG.info("%s\n", ptitle)
+                else:
+                    pull.edit(title=ptitle)
+                    LOG.debug("Updated pull-request title")
+            elif body:
+                if dry_run:
+                    LOG.info("Would edit body")
+                    LOG.info("%s\n", body)
+                else:
+                    pull.edit(body=body)
+                    LOG.debug("Updated pull-request body")
 
             if comment:
-                # FIXME(jd) we should be able to comment directly on a PR
-                # without getting it as an issue but pygithub does not allow
-                # that yet
-                repo_to_fork.get_issue(pull.number).create_comment(comment)
-                LOG.debug("Commented: \"%s\"", comment)
+                if dry_run:
+                    LOG.info("Would comment: \"%s\"", comment)
+                else:
+                    # FIXME(jd) we should be able to comment directly on a PR
+                    # without getting it as an issue but pygithub does not
+                    # allow that yet
+                    repo_to_fork.get_issue(pull.number).create_comment(comment)
+                    LOG.debug("Commented: \"%s\"", comment)
 
             if labels:
-                LOG.debug("Adding labels %s", labels)
-                pull.add_to_labels(*labels)
+                if dry_run:
+                    LOG.info("Would add labels %s", labels)
+                else:
+                    LOG.debug("Adding labels %s", labels)
+                    pull.add_to_labels(*labels)
+
+            LOG.info("Pull-request updated: %s", pull.html_url)
     else:
-        if dry_run:
-            LOG.info("Pull-request would be created.")
-            return
         # Create a pull request
-        if force_editor or not (title and message):
-            nb_of_commits, git_title, git_message = git_get_title_and_message(
-                "%s/%s" % (target_remote, target_branch), branch)
+        if not title or not message:
             title = title or git_title
             message = message or git_message
-            # Do not run an editor if there's only one commit or if both
-            # title and message were specified
-            if (force_editor or
-               ((not title or not message) and nb_of_commits > 1) or
-               (((title and not message) or
-                 (message and not title)) and nb_of_commits == 1)):
-                title, message = edit_title_and_message(title, message)
+            title, message = edit_title_and_message(title, message)
 
         if title is None:
             LOG.critical("Pull-request message is empty, aborting")
             return 40
 
+        if dry_run:
+            LOG.info("Pull-request would be created.")
+            LOG.info("Title: %s", title)
+            LOG.info("Body: %s", message)
+            return
+
         try:
             pull = repo_to_fork.create_pull(base=target_branch,
                                             head=head,
@@ -598,9 +624,6 @@ def fork_and_push_pull_request(g, hosttype, repo_to_fork, rebase,
             LOG.debug("Adding labels %s", labels)
             pull.add_to_labels(*labels)
 
-    if tag_previous_revision:
-        preserve_older_revision(branch, remote_to_push)
-
 
 def _format_github_exception(action, exc):
     url = exc.data.get("documentation_url", "GitHub documentation")
@@ -662,23 +685,10 @@ def build_parser():
     git_config_add_argument(parser, "--no-rebase", "-R",
                             action="store_true",
                             help="Don't rebase branch before pushing.")
-    git_config_add_argument(
-        parser,
-        "--force-editor",
-        action="store_true",
-        default=False,
-        help="Force editor to run to edit pull-request message.")
     parser.add_argument(
         "--comment", "-C",
         help="Comment to publish when updating the pull-request"
     )
-    git_config_add_argument(
-        parser,
-        "--tag-previous-revision",
-        action="store_true",
-        default=False,
-        help="Preserve older revision when pushing"
-    )
     group = parser.add_mutually_exclusive_group()
     git_config_add_argument(
         group,
@@ -728,10 +738,8 @@ def main():
             message=args.message,
             comment=args.comment,
             rebase=not args.no_rebase,
-            force_editor=args.force_editor,
             download=args.download,
             download_setup=args.download_setup,
-            tag_previous_revision=args.tag_previous_revision,
             fork=args.fork,
             setup_only=args.setup_only,
             branch_prefix=args.branch_prefix,
